// PaulK Libus
// Copyright (C) 2011 Paul Kocialkowski, GNU GPLv3+
// 
// This program is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// You should have received a copy of the GNU General Public License
// along with this program.  If not, see <http://www.gnu.org/licenses/>.

// The bus logo is based on: <https://secure.wikimedia.org/wikipedia/commons/wiki/File:Aiga_bus_on_green_circle.svg>, Public Domain
// The refresh icon comes from the Tango Icon Library <http://tango.freedesktop.org/>, Public Domain.

package fr.paulk.libus;

import android.app.Activity;
import android.os.Bundle;

import android.text.format.Time;
import android.widget.TextView;
import android.text.format.DateFormat;
import android.view.Menu;
import android.view.MenuInflater;
import android.view.MenuItem;
import android.graphics.Typeface;

//import java.lang.Math;

/* NOTE: This package has no interest to be used without any modification since it is configured to display the bus schedule of the bus I usually use in the city where I live.
         But it can be used to make your own app with your own bus schedule. */

public class PaulKLibus extends Activity
{
	private static final int PL_ROUTE_M2L = 1;
	private static final int PL_ROUTE_L2M = 2;

	private static int pl_route=0;

	/* Arrays with bus schedules */

	private int pl_libus_ligne_1_m2l_m[][] = { 
		{  6, 38 }, 
		{  7, 13 },
		{  7, 58 },
		{  8, 33 },
		{  9, 18 },
		{  9, 53 },
		{ 10, 38 },
		{ 11, 13 },
		{ 11, 58 },
		{ 13, 23 },
		{ 14,  8 },
		{ 14, 43 },
		{ 15, 28 },
		{ 16,  3 },
		{ 16, 58 },
		{ 17, 43 },
		{ 18, 18 },
		{ 19,  3 }
	};

	private int pl_libus_ligne_1_m2l_l[][] = { 
		{  6, 55 }, 
		{  7, 30 },
		{  8, 15 },
		{  8, 50 },
		{  9, 35 },
		{ 10, 10 },
		{ 10, 55 },
		{ 11, 30 },
		{ 12, 15 },
		{ 13, 40 },
		{ 14, 25 },
		{ 15,  0 },
		{ 15, 45 },
		{ 16, 20 },
		{ 17, 15 },
		{ 18, 00 },
		{ 18, 35 },
		{ 19, 20 }
	};

	private int pl_libus_ligne_1_l2m_l[][] = { 
		{  6, 45 }, 
		{  7, 30 },
		{  8, 05 },
		{  8, 50 },
		{  9, 25 },
		{ 10, 10 },
		{ 10, 45 },
		{ 11, 30 },
		{ 12, 15 },
		{ 13, 40 },
		{ 14, 15 },
		{ 15,  0 },
		{ 15, 35 },
		{ 16, 20 },
		{ 17, 15 },
		{ 17, 50 },
		{ 18, 35 },
		{ 19, 20 }
	};

	private int pl_libus_ligne_1_l2m_m[][] = { 
		{  7,  0 }, 
		{  7, 45 },
		{  8, 20 },
		{  9,  5 },
		{  9, 40 },
		{ 10, 25 },
		{ 11,  0 },
		{ 11, 45 },
		{ 12, 30 },
		{ 13, 55 },
		{ 14, 30 },
		{ 15, 15 },
		{ 15, 50 },
		{ 16, 33 },
		{ 17, 30 },
		{ 18, 05 },
		{ 18, 50 },
		{ 19, 35 }
	};

	private int pl_libus_ligne_2_m2l_m[][] = { 
		{  6, 32 }, 
		{  7, 17 },
		{  8,  2 },
		{  8, 47 },
		{  9, 32 },
		{ 10, 17 },
		{ 11,  2 },
		{ 11, 57 },
		{ 12, 45 },
		{ 13, 22 },
		{ 14,  7 },
		{ 14, 52 },
		{ 15, 37 },
		{ 16, 22 },
		{ 17,  7 },
		{ 17, 52 },
		{ 18, 37 },
		{ 19, 17 }
	};

	private int pl_libus_ligne_2_m2l_l[][] = { 
		{  6, 45 }, 
		{  7, 35 },
		{  8, 16 },
		{  9,  0 },
		{  9, 45 },
		{ 10, 30 },
		{ 11, 15 },
		{ 12, 15 },
		{ 12, 58 },
		{ 13, 35 },
		{ 14, 20 },
		{ 15,  5 },
		{ 15, 50 },
		{ 16, 35 },
		{ 17, 20 },
		{ 18,  5 },
		{ 18, 50 },
		{ 19, 30 }
	};

	private int pl_libus_ligne_2_l2m_l[][] = { 
		{  6, 45 }, 
		{  7, 30 },
		{  8, 15 },
		{  9,  0 },
		{  9, 45 },
		{ 10, 30 },
		{ 11, 15 },
		{ 12, 15 },
		{ 13, 35 },
		{ 14, 20 },
		{ 15,  5 },
		{ 15, 50 },
		{ 16, 35 },
		{ 17, 20 },
		{ 18,  5 },
		{ 18, 50 },
		{ 19, 30 }
	};

	private int pl_libus_ligne_2_l2m_m[][] = { 
		{  6, 58 }, 
		{  7, 43 },
		{  8, 28 },
		{  9, 13 },
		{  9, 58 },
		{ 10, 43 },
		{ 11, 28 },
		{ 12, 28 },
		{ 13, 48 },
		{ 14, 33 },
		{ 15, 18 },
		{ 16,  3 },
		{ 16, 48 },
		{ 17, 33 },
		{ 18, 18 },
		{ 19,  3 },
		{ 19, 43 }
	};

	private int pl_libus_ligne_3_m2l_m[][] = { 
		{  7, 10 }, 
		{  8, 10 },
		{  9,  5 },
		{  9, 50 },
		{ 10, 55 },
		{ 11, 45 },
		{ 13, 15 },
		{ 14, 10 },
		{ 14, 55 },
		{ 15, 50 },
		{ 16, 55 },
		{ 17, 50 }
	};

	private int pl_libus_ligne_3_m2l_l[][] = { 
		{  7, 35 }, 
		{  8, 35 },
		{  9, 30 },
		{ 10, 15 },
		{ 11, 20 },
		{ 12, 10 },
		{ 13, 40 },
		{ 14, 35 },
		{ 15, 20 },
		{ 16, 15 },
		{ 17, 20 },
		{ 18, 15 }
	};

	private int pl_libus_ligne_3_l2m_l[][] = { 
		{  7, 35 }, 
		{  8, 35 },
		{  9, 15 },
		{ 10, 20 },
		{ 11,  5 },
		{ 12, 15 },
		{ 13, 40 },
		{ 14, 25 },
		{ 15, 20 },
		{ 16,  5 },
		{ 17, 15 },
		{ 18,  5 }
	};

	private int pl_libus_ligne_3_l2m_m[][] = { 
		{  8,  0 }, 
		{  9,  0 },
		{  9, 40 },
		{ 10, 45 },
		{ 11, 30 },
		{ 12, 40 },
		{ 14,  5 },
		{ 14, 50 },
		{ 15, 45 },
		{ 16, 30 },
		{ 17, 40 },
		{ 18, 30 }
	};

	/* Computate the next bus */
	private int pl_get_next_libus(Time time_now, int pl_table_s[][])
	{
		for(int i=0 ; i < pl_table_s.length ; i++)
		{
			if(time_now.hour <= pl_table_s[i][0])
			{
				if(time_now.minute < pl_table_s[i][1] || time_now.hour < pl_table_s[i][0])
				{
					return i;
				}
			}
		}

		return -1;
	}

	/* return the formatted time (ex: 09:12) */
	private String pl_get_ftime(int time_hour, int time_min)
	{
		String pl_get_ftime_return = new String();

		if(time_hour < 10)
			pl_get_ftime_return += "0" + String.valueOf(time_hour);
		else
			pl_get_ftime_return += String.valueOf(time_hour);

		pl_get_ftime_return += ":";

		if(time_min < 10)
			pl_get_ftime_return += "0" + String.valueOf(time_min);
		else
			pl_get_ftime_return += String.valueOf(time_min);

		return pl_get_ftime_return;
	}

	/* Add the infos to the TextView. Computate infos from the time in second format (starting from midnight). */
	private void pl_infos_append(TextView pl_infos, int pl_ligne_start_time, int pl_ligne_end_time, Time time_now)
	{
		pl_infos.setText("");

		pl_infos.append(
			getString(R.string.pl_next_libus) + " " + pl_get_ftime(
				(pl_ligne_start_time - (pl_ligne_start_time % 60)) / 60, 
				pl_ligne_start_time % 60
			)
		);

		pl_infos.append("\n");

		pl_infos.append(
			getString(R.string.pl_in_time) + " " + pl_get_ftime(
				(
					(pl_ligne_start_time - (time_now.hour * 60 + time_now.minute)) - 
					(pl_ligne_start_time - (time_now.hour * 60 + time_now.minute)) % 60 
				) / 60, 
				(pl_ligne_start_time - (time_now.hour * 60 + time_now.minute)) % 60
			)
		);

		pl_infos.append("\n");

		pl_infos.append(
			getString(R.string.pl_end_time) + " " + pl_get_ftime(
				(pl_ligne_end_time - (pl_ligne_end_time % 60)) / 60, 
				pl_ligne_end_time % 60
			)
		);

		pl_infos.append("\n");

		pl_infos.append(
			getString(R.string.pl_route_time) + " " + pl_get_ftime(
				(
					(pl_ligne_end_time) - (pl_ligne_end_time) % 60 
				) / 60 - 
			
				(
					(pl_ligne_start_time) - (pl_ligne_start_time) % 60 
				) / 60,
				pl_ligne_end_time % 60 - 
				pl_ligne_start_time % 60
			)
		);
	}

	private void pl_construct_m2l()
	{
		int pl_next_libus_id=-1;
		int pl_next_libus_mtime=0;
		int pl_next_libus_ltime=0;

		TextView pl_infos;

		/* Getting time */
		Time time_now = new Time();
		time_now.setToNow();

		/* Setting the route */
		pl_route=PL_ROUTE_M2L;

		/* Setting the route in the TextView */
		TextView pl_route = (TextView) findViewById(R.id.pl_route);
		pl_route.setText(getString(R.string.pl_route_label) + " " + getString(R.string.m2l));

		/* PaulK Libus Ligne 1 stuff */
	
		pl_next_libus_id=pl_get_next_libus(time_now, this.pl_libus_ligne_1_m2l_m);
		pl_infos=(TextView) findViewById(R.id.pl_1_infos);

		if(pl_next_libus_id != -1)
		{
			pl_next_libus_mtime=pl_libus_ligne_1_m2l_m[pl_next_libus_id][0] * 60 + pl_libus_ligne_1_m2l_m[pl_next_libus_id][1];
			pl_next_libus_ltime=pl_libus_ligne_1_m2l_l[pl_next_libus_id][0] * 60 + pl_libus_ligne_1_m2l_l[pl_next_libus_id][1];

			pl_infos_append(
				pl_infos, 
				pl_next_libus_mtime,
				pl_next_libus_ltime,
				time_now
			);
		}
		else
				pl_infos.setText(getString(R.string.noinfo));

		/* PaulK Libus Ligne 2 stuff */
	
		pl_next_libus_id=pl_get_next_libus(time_now, this.pl_libus_ligne_2_m2l_m);
		pl_infos=(TextView) findViewById(R.id.pl_2_infos);

		if(pl_next_libus_id != -1)
		{
			pl_next_libus_mtime=pl_libus_ligne_2_m2l_m[pl_next_libus_id][0] * 60 + pl_libus_ligne_2_m2l_m[pl_next_libus_id][1];
			pl_next_libus_ltime=pl_libus_ligne_2_m2l_l[pl_next_libus_id][0] * 60 + pl_libus_ligne_2_m2l_l[pl_next_libus_id][1];

			pl_infos_append(
				pl_infos, 
				pl_next_libus_mtime,
				pl_next_libus_ltime,
				time_now
			);
		}
		else
				pl_infos.setText(getString(R.string.noinfo));

		/* PaulK Libus Ligne 3 stuff */
	
		pl_next_libus_id=pl_get_next_libus(time_now, this.pl_libus_ligne_3_m2l_m);
		pl_infos=(TextView) findViewById(R.id.pl_3_infos);

		if(pl_next_libus_id != -1)
		{
			pl_next_libus_mtime=pl_libus_ligne_3_m2l_m[pl_next_libus_id][0] * 60 + pl_libus_ligne_3_m2l_m[pl_next_libus_id][1];
			pl_next_libus_ltime=pl_libus_ligne_3_m2l_l[pl_next_libus_id][0] * 60 + pl_libus_ligne_3_m2l_l[pl_next_libus_id][1];

			pl_infos_append(
				pl_infos, 
				pl_next_libus_mtime,
				pl_next_libus_ltime,
				time_now
			);
		}
		else
				pl_infos.setText(getString(R.string.noinfo));

	}

	private void pl_construct_l2m()
	{
		int pl_next_libus_id=-1;
		int pl_next_libus_mtime=0;
		int pl_next_libus_ltime=0;

		TextView pl_infos;

		/* Getting time */
		Time time_now = new Time();
		time_now.setToNow();

		/* Setting the route */
		pl_route=PL_ROUTE_L2M;

		/* Setting the route in the TextView */
		TextView pl_route = (TextView) findViewById(R.id.pl_route);
		pl_route.setText(getString(R.string.pl_route_label) + " " + getString(R.string.l2m));

		/* PaulK Libus Ligne 1 stuff */
	
		pl_next_libus_id=pl_get_next_libus(time_now, this.pl_libus_ligne_1_l2m_l);
		pl_infos=(TextView) findViewById(R.id.pl_1_infos);

		if(pl_next_libus_id != -1)
		{
			pl_next_libus_mtime=pl_libus_ligne_1_l2m_m[pl_next_libus_id][0] * 60 + pl_libus_ligne_1_l2m_m[pl_next_libus_id][1];
			pl_next_libus_ltime=pl_libus_ligne_1_l2m_l[pl_next_libus_id][0] * 60 + pl_libus_ligne_1_l2m_l[pl_next_libus_id][1];

			pl_infos_append(
				pl_infos,
				pl_next_libus_ltime,
				pl_next_libus_mtime,
				time_now
			);
		}
		else
				pl_infos.setText(getString(R.string.noinfo));

		/* PaulK Libus Ligne 2 stuff */
	
		pl_next_libus_id=pl_get_next_libus(time_now, this.pl_libus_ligne_2_l2m_l);
		pl_infos=(TextView) findViewById(R.id.pl_2_infos);

		if(pl_next_libus_id != -1)
		{
			pl_next_libus_mtime=pl_libus_ligne_2_l2m_m[pl_next_libus_id][0] * 60 + pl_libus_ligne_2_l2m_m[pl_next_libus_id][1];
			pl_next_libus_ltime=pl_libus_ligne_2_l2m_l[pl_next_libus_id][0] * 60 + pl_libus_ligne_2_l2m_l[pl_next_libus_id][1];

			pl_infos_append(
				pl_infos,
				pl_next_libus_ltime,
				pl_next_libus_mtime,
				time_now
			);
		}
		else
				pl_infos.setText(getString(R.string.noinfo));

		/* PaulK Libus Ligne 3 stuff */

		pl_next_libus_id=pl_get_next_libus(time_now, this.pl_libus_ligne_3_l2m_l);
		pl_infos=(TextView) findViewById(R.id.pl_3_infos);

		if(pl_next_libus_id != -1)
		{
			pl_next_libus_mtime=pl_libus_ligne_3_l2m_m[pl_next_libus_id][0] * 60 + pl_libus_ligne_3_l2m_m[pl_next_libus_id][1];
			pl_next_libus_ltime=pl_libus_ligne_3_l2m_l[pl_next_libus_id][0] * 60 + pl_libus_ligne_3_l2m_l[pl_next_libus_id][1];

			pl_infos_append(
				pl_infos,
				pl_next_libus_ltime,
				pl_next_libus_mtime,
				time_now
			);
		}
		else
				pl_infos.setText(getString(R.string.noinfo));

	}

	/* Init the layout (sets the font for titles) */
	private void pl_layout_init()
	{
		Typeface pl_font = Typeface.createFromAsset(getAssets(), "Ubuntu-M.ttf");

		TextView pl_title = (TextView) findViewById(R.id.pl_title);
		pl_title.setTypeface(pl_font);

		TextView pl_1_title = (TextView) findViewById(R.id.pl_1_title);
		pl_1_title.setTypeface(pl_font);

		TextView pl_2_title = (TextView) findViewById(R.id.pl_2_title);
		pl_2_title.setTypeface(pl_font);

		TextView pl_3_title = (TextView) findViewById(R.id.pl_3_title);
		pl_3_title.setTypeface(pl_font);
	}

	/* Select the route to use with the time or the last choice */
	private void pl_route_init()
	{
		Time time_now = new Time();
		time_now.setToNow();

		switch(pl_route)
		{
			case PL_ROUTE_M2L:
				pl_construct_m2l();
			break;
			case PL_ROUTE_L2M:
				pl_construct_l2m();
			break;
			default:
				if(time_now.hour < 12)
					pl_construct_m2l();
				else
					pl_construct_l2m();
			break;
		}
	}
	
	/* This function is called when the program starts */
	@Override
	public void onCreate(Bundle savedInstanceState)
	{
		super.onCreate(savedInstanceState);

		setContentView(R.layout.main); 
		pl_layout_init();
		pl_route_init();
	}

	/* Creates the options menu */
	@Override
	public boolean onCreateOptionsMenu(Menu menu)
	{
		MenuInflater inflater = getMenuInflater();
		inflater.inflate(R.menu.menu, menu);
		return true;
	}

	/* Handles the selected item */
	@Override
	public boolean onOptionsItemSelected(MenuItem item) 
	{
		switch (item.getItemId())
		{
			case R.id.m2l:
				pl_construct_m2l();
			return true;
			case R.id.l2m:
				pl_construct_l2m();
			return true;
			case R.id.refresh:
				pl_route_init();
			return true;
		}

		return false;
	}
}
